"""Draw color arrow glyphs"""
from vedo import *
import numpy as np

# Create two spheres with different radii, wireframes, 
# and colors, and set the position of one of them
s1 = Sphere(r=10, res=8).wireframe().c('white')
s2 = Sphere(r=20, res=8).wireframe().c('white',0.1).pos(0,4,0)

# Get the coordinates of the vertices of each sphere
coords1 = s1.points() 
coords2 = s2.points()

# --- color can be a colormap which maps arrow sizes
# Define a title for the first set of arrows,
#  and create an Arrows object with coordinates and a colormap for color
t1 = 'Color arrows by size\nusing a color map'
a1 = Arrows(coords1, coords2, c='coolwarm', alpha=0.4).add_scalarbar(c='w')

# --- get a list of random rgb colors
# Generate a list of random RGB colors for each arrow 
# based on an array of integers, and define a title for the second set of arrows
nrs = np.random.randint(0, 10, len(coords1))
cols = get_color(nrs)
t2 = 'Color arrows by an array\nand scale them by half'
a2 = Arrows(coords1, coords2, c=cols)

# Display two groups of objects on two renderers: the two spheres, 
# the Arrows object with a colormap for color and a scalar bar, 
# and the title for the first set of arrows on one renderer; 
# the two spheres, the Arrows object with random RGB colors, 
# and the title for the second set of arrows on another renderer
show([(s1, s2, a1, t1), (s1, s2, a2, t2)], N=2, bg='bb', bg2='lb').close()



from vedo import Grid, Tensors, show

domain = Grid(res=[5,5], c='gray')

# Generate random attributes on this mesh
domain.generate_random_data()

ts = Tensors(domain, scale=0.1)
ts.print()

show(domain, ts).close()




"""Fit circles analytically to measure
the signed curvature of a line"""
from vedo import *

shape = Spline([[1.0, 2.0, -1.0],
                [1.5, 0.0,  0.4],
                [2.0, 4.0,  0.5],
                [4.0, 1.5, -0.3]], res=200)

n = 5  # nr. of points to use for the fit
npt = shape.npoints

points = shape.points()
fitpts, circles, curvs = [], [], [0]*npt

for i in range(n, npt - n-1):
    pts = points[i-n:i+n]
    center, R, normal = fit_circle(pts)
    z = cross(pts[-1]-pts[0], center-pts[0])[2]
    curvs[i] = sqrt(1/R) * z/abs(z)
    if R < 0.75:
        circle = Circle(center, r=R).wireframe().orientation(normal)
        circles.append(circle)
        fitpts.append(center)

shape.lw(8).cmap('coolwarm', curvs).add_scalarbar3d(title=':pm1/:sqrtR', c='w')
show(shape, circles, Points(fitpts), __doc__, axes=1, bg='bb').close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import sys
import time

import numpy as np

try:
    import vedo.vtkclasses as vtk
except ImportError:
    import vtkmodules.all as vtk

import vedo

__docformat__ = "google"

__doc__ = """
Colors definitions and printing methods.

![](https://vedo.embl.es/images/basic/colorcubes.png)
"""

__all__ = [
    "printc",
    "printd",
    "get_color",
    "get_color_name",
    "color_map",
    "build_palette",
    "build_lut",
    "getColor",  # deprecated will disappear
    "colorMap",  # deprecated
    "buildLUT",  # deprecated
]


try:
    import matplotlib.cm as _cm_mpl
    _has_matplotlib = True
    cmaps = {}
except ModuleNotFoundError:
    from vedo.cmaps import cmaps
    _has_matplotlib = False
    # see below, this is dealt with in color_map()

#########################################################
# handy global shortcuts for terminal printing
# Ex.: print(colors.red + "hello" + colors.reset)
#########################################################
red = "\x1b[1m\x1b[31;1m"
green = "\x1b[1m\x1b[32;1m"
yellow = "\x1b[1m\x1b[33;1m"
blue = "\x1b[1m\x1b[34;1m"
reset = "\x1b[0m"


#########################################################
# basic color schemes
#########################################################
colors = {
    # order kind of matters because of pyplot.plot()
    "blue9": "#a8cbfe",  # bootstrap5 colors
    "blue8": "#81b4fe",
    "blue7": "#5a9cfe",
    "blue6": "#3485fd",
    "blue5": "#0d6efd",
    "blue4": "#0b5cd5",
    "blue3": "#094bac",
    "blue2": "#073984",
    "blue1": "#05285b",
    "indigo9": "#c8a9fa",
    "indigo8": "#af83f8",
    "indigo7": "#975cf6",
    "indigo6": "#7e36f4",
    "indigo5": "#6610f2",
    "indigo4": "#560dcb",
    "indigo3": "#450ba5",
    "indigo2": "#35087e",
    "indigo1": "#250657",
    "purple9": "#cbbbe9",
    "purple8": "#b49ddf",
    "purple7": "#9d7ed5",
    "purple6": "#8660cb",
    "purple5": "#6f42c1",
    "purple4": "#5d37a2",
    "purple3": "#4b2d83",
    "purple2": "#3a2264",
    "purple1": "#281845",
    "pink9": "#f0b6d3",
    "pink8": "#ea95bf",
    "pink7": "#e374ab",
    "pink6": "#dd5498",
    "pink5": "#d63384",
    "pink4": "#b42b6f",
    "pink3": "#92235a",
    "pink2": "#6f1b45",
    "pink1": "#4d1230",
    "red9": "#f2b6bc",
    "red8": "#ed969e",
    "red7": "#e77681",
    "red6": "#e25563",
    "red5": "#dc3545",
    "red4": "#b92d3a",
    "red3": "#96242f",
    "red2": "#721c24",
    "red1": "#4f1319",
    "orange9": "#fed1aa",
    "orange8": "#febc85",
    "orange7": "#fea75f",
    "orange6": "#fd933a",
    "orange5": "#fd7e14",
    "orange4": "#d56a11",
    "orange3": "#ac560e",
    "orange2": "#84420a",
    "orange1": "#5b2d07",
    "yellow9": "#ffe9a6",
    "yellow8": "#ffdf7e",
    "yellow7": "#ffd556",
    "yellow6": "#ffcb2f",
    "yellow5": "#ffc107",
    "yellow4": "#d6a206",
    "yellow3": "#ad8305",
    "yellow2": "#856404",
    "yellow1": "#5c4503",
    "green9": "#b2dfbc",
    "green8": "#8fd19e",
    "green7": "#6dc381",
    "green6": "#4ab563",
    "green5": "#28a745",
    "green4": "#228c3a",
    "green3": "#1b722f",
    "green2": "#155724",
    "green1": "#0e3c19",
    "teal9": "#afecda",
    "teal8": "#8be3c9",
    "teal7": "#67dab8",
    "teal6": "#44d2a8",
    "teal5": "#20c997",
    "teal4": "#1ba97f",
    "teal3": "#168967",
    "teal2": "#11694f",
    "teal1": "#0c4836",
    "cyan9": "#abdee5",
    "cyan8": "#86cfda",
    "cyan7": "#61c0cf",
    "cyan6": "#3cb1c3",
    "cyan5": "#17a2b8",
    "cyan4": "#13889b",
    "cyan3": "#106e7d",
    "cyan2": "#0c5460",
    "cyan1": "#083a42",
    "gray9": "#f8f9fa",
    "gray8": "#e9edef",
    "gray7": "#dee2e6",
    "gray6": "#ced4da",
    "gray5": "#adb5bd",
    "gray4": "#6c757d",
    "gray3": "#495057",
    "gray2": "#343a40",
    "gray1": "#212529",
    "aliceblue": "#F0F8FF",  # matplotlib scheme
    "antiquewhite": "#FAEBD7",
    "aqua": "#00FFFF",
    "aquamarine": "#7FFFD4",
    "azure": "#F0FFFF",
    "beige": "#F5F5DC",
    "bisque": "#FFE4C4",
    "black": "#000000",
    "blanchedalmond": "#FFEBCD",
    "blue": "#0f00fb",  # "0000FF",
    "blueviolet": "#8A2BE2",
    "brown": "#A52A2A",
    "burlywood": "#DEB887",
    "cadetblue": "#5F9EA0",
    "chartreuse": "#7FFF00",
    "chocolate": "#D2691E",
    "coral": "#FF7F50",
    "cornflowerblue": "#6495ED",
    "cornsilk": "#FFF8DC",
    "crimson": "#DC143C",
    "cyan": "#00FFFF",
    "darkblue": "#00008B",
    "darkcyan": "#008B8B",
    "darkgoldenrod": "#B8860B",
    "darkgray": "#A9A9A9",
    "darkgreen": "#006400",
    "darkkhaki": "#BDB76B",
    "darkmagenta": "#8B008B",
    "darkolivegreen": "#556B2F",
    "darkorange": "#FF8C00",
    "darkorchid": "#9932CC",
    "darkred": "#8B0000",
    "darksalmon": "#E9967A",
    "darkseagreen": "#8FBC8F",
    "darkslateblue": "#483D8B",
    "darkslategray": "#2F4F4F",
    "darkturquoise": "#00CED1",
    "darkviolet": "#9400D3",
    "deeppink": "#FF1493",
    "deepskyblue": "#00BFFF",
    "dimgray": "#696969",
    "dodgerblue": "#1E90FF",
    "firebrick": "#B22222",
    "floralwhite": "#FFFAF0",
    "forestgreen": "#228B22",
    "fuchsia": "#FF00FF",
    "gainsboro": "#DCDCDC",
    "ghostwhite": "#F8F8FF",
    "gold": "#FFD700",
    "goldenrod": "#DAA520",
    "gray": "#808080",
    "green": "#047f10",  # "#008000",
    "greenyellow": "#ADFF2F",
    "honeydew": "#F0FFF0",
    "hotpink": "#FF69B4",
    "indianred": "#CD5C5C",
    "indigo": "#4B0082",
    "ivory": "#FFFFF0",
    "khaki": "#F0E68C",
    "lavender": "#E6E6FA",
    "lavenderblush": "#FFF0F5",
    "lawngreen": "#7CFC00",
    "lemonchiffon": "#FFFACD",
    "lightblue": "#ADD8E6",
    "lightcoral": "#F08080",
    "lightcyan": "#E0FFFF",
    "lightgray": "#D3D3D3",
    "lightgreen": "#90EE90",
    "lightpink": "#FFB6C1",
    "lightsalmon": "#FFA07A",
    "lightseagreen": "#20B2AA",
    "lightskyblue": "#87CEFA",
    "lightsteelblue": "#B0C4DE",
    "lightyellow": "#FFFFE0",
    "lime": "#00FF00",
    "limegreen": "#32CD32",
    "linen": "#FAF0E6",
    "magenta": "#FF00FF",
    "maroon": "#800000",
    "mediumaquamarine": "#66CDAA",
    "mediumblue": "#0000CD",
    "mediumorchid": "#BA55D3",
    "mediumpurple": "#9370DB",
    "mediumseagreen": "#3CB371",
    "mediumslateblue": "#7B68EE",
    "mediumspringgreen": "#00FA9A",
    "mediumturquoise": "#48D1CC",
    "mediumvioletred": "#C71585",
    "midnightblue": "#191970",
    "mintcream": "#F5FFFA",
    "mistyrose": "#FFE4E1",
    "moccasin": "#FFE4B5",
    "navajowhite": "#FFDEAD",
    "navy": "#000080",
    "oldlace": "#FDF5E6",
    "olive": "#808000",
    "olivedrab": "#6B8E23",
    "orange": "#FFA500",
    "orangered": "#FF4500",
    "orchid": "#DA70D6",
    "palegoldenrod": "#EEE8AA",
    "palegreen": "#98FB98",
    "paleturquoise": "#AFEEEE",
    "palevioletred": "#DB7093",
    "papayawhip": "#FFEFD5",
    "peachpuff": "#FFDAB9",
    "peru": "#CD853F",
    "pink": "#FFC0CB",
    "plum": "#DDA0DD",
    "powderblue": "#B0E0E6",
    "purple": "#800080",
    "rebeccapurple": "#663399",
    "red": "#fe1e1f",  # "#FF0000",
    "rosybrown": "#BC8F8F",
    "royalblue": "#4169E1",
    "saddlebrown": "#8B4513",
    "salmon": "#FA8072",
    "sandybrown": "#F4A460",
    "seagreen": "#2E8B57",
    "seashell": "#FFF5EE",
    "sienna": "#A0522D",
    "silver": "#C0C0C0",
    "skyblue": "#87CEEB",
    "slateblue": "#6A5ACD",
    "slategray": "#708090",
    "snow": "#FFFAFA",
    "blackboard": "#393939",
    "springgreen": "#00FF7F",
    "steelblue": "#4682B4",
    "tan": "#D2B48C",
    "teal": "#008080",
    "thistle": "#D8BFD8",
    "tomato": "#FF6347",
    "turquoise": "#40E0D0",
    "violet": "#EE82EE",
    "wheat": "#F5DEB3",
    "white": "#FFFFFF",
    "whitesmoke": "#F5F5F5",
    "yellow": "#ffff36",  # "#FFFF00",
    "yellowgreen": "#9ACD32",
}


color_nicks = {  # color nicknames
    "bb": "blackboard",
    "lb": "lightblue",  # light
    "lg": "lightgreen",
    "lr": "orangered",
    "lc": "lightcyan",
    "ls": "lightsalmon",
    "ly": "lightyellow",
    "dr": "darkred",  # dark
    "db": "darkblue",
    "dg": "darkgreen",
    "dm": "darkmagenta",
    "dc": "darkcyan",
    "ds": "darksalmon",
    "dv": "darkviolet",
    "b1": "blue1",  # bootstrap5 colors
    "b2": "blue2",
    "b3": "blue3",
    "b4": "blue4",
    "b5": "blue5",
    "b6": "blue6",
    "b7": "blue7",
    "b8": "blue8",
    "b9": "blue9",
    "i1": "indigo1",
    "i2": "indigo2",
    "i3": "indigo3",
    "i4": "indigo4",
    "i5": "indigo5",
    "i6": "indigo6",
    "i7": "indigo7",
    "i8": "indigo8",
    "i9": "indigo9",
    "p1": "purple1",
    "p2": "purple2",
    "p3": "purple3",
    "p4": "purple4",
    "p5": "purple5",
    "p6": "purple6",
    "p7": "purple7",
    "p8": "purple8",
    "p9": "purple9",
    "r1": "red1",
    "r2": "red2",
    "r3": "red3",
    "r4": "red4",
    "r5": "red5",
    "r6": "red6",
    "r7": "red7",
    "r8": "red8",
    "r9": "red9",
    "o1": "orange1",
    "o2": "orange2",
    "o3": "orange3",
    "o4": "orange4",
    "o5": "orange5",
    "o6": "orange6",
    "o7": "orange7",
    "o8": "orange8",
    "o9": "orange9",
    "y1": "yellow1",
    "y2": "yellow2",
    "y3": "yellow3",
    "y4": "yellow4",
    "y5": "yellow5",
    "y6": "yellow6",
    "y7": "yellow7",
    "y8": "yellow8",
    "y9": "yellow9",
    "g1": "green1",
    "g2": "green2",
    "g3": "green3",
    "g4": "green4",
    "g5": "green5",
    "g6": "green6",
    "g7": "green7",
    "g8": "green8",
    "g9": "green9",
    "k1": "gray1",
    "k2": "gray2",
    "k3": "gray3",
    "k4": "gray4",
    "k5": "gray5",
    "k6": "gray6",
    "k7": "gray7",
    "k8": "gray8",
    "k9": "gray9",
    "a": "aqua",
    "b": "blue",
    "c": "cyan",
    "d": "gold",
    "f": "fuchsia",
    "g": "green",
    "i": "indigo",
    "k": "black",
    "m": "magenta",
    "n": "navy",
    "l": "lavender",
    "o": "orange",
    "p": "purple",
    "r": "red",
    "s": "salmon",
    "t": "tomato",
    "v": "violet",
    "y": "yellow",
    "w": "white",
}


# available colormap names:
cmaps_names = (
    "Accent",
    "Accent_r",
    "Blues",
    "Blues_r",
    "BrBG",
    "BrBG_r",
    "BuGn",
    "BuGn_r",
    "BuPu",
    "BuPu_r",
    "CMRmap",
    "CMRmap_r",
    "Dark2",
    "Dark2_r",
    "GnBu",
    "GnBu_r",
    "Greens",
    "Greens_r",
    "Greys",
    "Greys_r",
    "OrRd",
    "OrRd_r",
    "Oranges",
    "Oranges_r",
    "PRGn",
    "PRGn_r",
    "Paired",
    "Paired_r",
    "Pastel1",
    "Pastel1_r",
    "Pastel2",
    "Pastel2_r",
    "PiYG",
    "PiYG_r",
    "PuBu",
    "PuBuGn",
    "PuBuGn_r",
    "PuBu_r",
    "PuOr",
    "PuOr_r",
    "PuRd",
    "PuRd_r",
    "Purples",
    "Purples_r",
    "RdBu",
    "RdBu_r",
    "RdGy",
    "RdGy_r",
    "RdPu",
    "RdPu_r",
    "RdYlBu",
    "RdYlBu_r",
    "RdYlGn",
    "RdYlGn_r",
    "Reds",
    "Reds_r",
    "Set1",
    "Set1_r",
    "Set2",
    "Set2_r",
    "Set3",
    "Set3_r",
    "Spectral",
    "Spectral_r",
    "Wistia",
    "Wistia_r",
    "YlGn",
    "YlGnBu",
    "YlGnBu_r",
    "YlGn_r",
    "YlOrBr",
    "YlOrBr_r",
    "YlOrRd",
    "YlOrRd_r",
    "afmhot",
    "afmhot_r",
    "autumn",
    "autumn_r",
    "binary",
    "binary_r",
    "bone",
    "bone_r",
    "brg",
    "brg_r",
    "bwr",
    "bwr_r",
    "cividis",
    "cividis_r",
    "cool",
    "cool_r",
    "coolwarm",
    "coolwarm_r",
    "copper",
    "copper_r",
    "cubehelix",
    "cubehelix_r",
    "flag",
    "flag_r",
    "gist_earth",
    "gist_earth_r",
    "gist_gray",
    "gist_gray_r",
    "gist_heat",
    "gist_heat_r",
    "gist_ncar",
    "gist_ncar_r",
    "gist_rainbow",
    "gist_rainbow_r",
    "gist_stern",
    "gist_stern_r",
    "gist_yarg",
    "gist_yarg_r",
    "gnuplot",
    "gnuplot2",
    "gnuplot2_r",
    "gnuplot_r",
    "gray_r",
    "hot",
    "hot_r",
    "hsv",
    "hsv_r",
    "inferno",
    "inferno_r",
    "jet",
    "jet_r",
    "magma",
    "magma_r",
    "nipy_spectral",
    "nipy_spectral_r",
    "ocean",
    "ocean_r",
    "pink_r",
    "plasma",
    "plasma_r",
    "prism",
    "prism_r",
    "rainbow",
    "rainbow_r",
    "seismic",
    "seismic_r",
    "spring",
    "spring_r",
    "summer",
    "summer_r",
    "tab10",
    "tab10_r",
    "tab20",
    "tab20_r",
    "tab20b",
    "tab20b_r",
    "tab20c",
    "tab20c_r",
    "terrain",
    "terrain_r",
    "twilight",
    "twilight_r",
    "twilight_shifted",
    "twilight_shifted_r",
    "viridis",
    "viridis_r",
    "winter",
    "winter_r",
)


# default color palettes when using an index
palettes = (
    (
        (1.0, 0.832, 0.000),  # gold
        (0.960, 0.509, 0.188),
        (0.901, 0.098, 0.194),
        (0.235, 0.85, 0.294),
        (0.46, 0.48, 0.000),
        (0.274, 0.941, 0.941),
        (0.0, 0.509, 0.784),
        (0.1, 0.1, 0.900),
        (0.902, 0.7, 1.000),
        (0.941, 0.196, 0.901),
    ),
    (
        (1.0, 0.832, 0),    # gold
        (0.59, 0.0, 0.09),  # dark red
        (0.5, 0.5, 0),      # yellow-green
        (0.0, 0.66, 0.42),  # green blue
        (0.5, 1.0, 0.0),    # green
        (0.0, 0.18, 0.65),  # blue
        (0.4, 0.0, 0.4),    # plum
        (0.4, 0.0, 0.6),
        (0.2, 0.4, 0.6),
        (0.1, 0.3, 0.2),
    ),
    (
        (0.010, 0.0706, 0.098),  #  -> black
        (0.0196, 0.369, 0.447),
        (0.0745, 0.573, 0.584),
        (0.584, 0.820, 0.741),
        (0.914, 0.847, 0.663),
        (0.929, 0.616, 0.149),
        (0.788, 0.412, 0.110),
        (0.729, 0.259, 0.0902),
        (0.678, 0.153, 0.110),
        (0.604, 0.153, 0.165),  #  -> red3
    ),
    (
        (0.345, 0.188, 0.071),  #  -> orange1
        (0.498, 0.314, 0.161),
        (0.573, 0.404, 0.239),
        (0.651, 0.545, 0.400),
        (0.714, 0.678, 0.569),
        (0.761, 0.773, 0.671),
        (0.643, 0.675, 0.533),
        (0.396, 0.427, 0.298),
        (0.255, 0.282, 0.204),
        (0.200, 0.239, 0.165),  #  -> blackboard
    ),
    (
        (0.937, 0.969, 0.820),  #  -> beige
        (0.729, 0.851, 0.714),
        (0.671, 0.639, 0.396),
        (0.447, 0.180, 0.180),
        (0.259, 0.055, 0.082),  #  -> red1
        (0.937, 0.969, 0.820),  #  -> beige
        (0.729, 0.851, 0.714),
        (0.671, 0.639, 0.396),
        (0.447, 0.180, 0.180),
        (0.259, 0.055, 0.082),  #  -> red1
    ),
    (
        (0.933, 0.298, 0.443),  #  -> red6
        (0.996, 0.824, 0.431),
        (0.082, 0.835, 0.631),
        (0.094, 0.537, 0.690),
        (0.035, 0.231, 0.294),  #  -> cyan1
        (0.933, 0.298, 0.443),  #  -> red6
        (0.996, 0.824, 0.431),
        (0.082, 0.835, 0.631),
        (0.094, 0.537, 0.690),
        (0.035, 0.231, 0.294),  #  -> cyan1
    ),
)


emoji = {
    ":bomb:": "\U0001F4A5",
    ":sparks:": "\U00002728",
    ":thumbup:": "\U0001F44D",
    ":target:": "\U0001F3AF",
    ":save:": "\U0001F4BE",
    ":noentry:": "\U000026D4",
    ":video:": "\U0001F4FD",
    ":lightning:": "\U000026A1",
    ":camera:": "\U0001F4F8",
    ":times:": "\U0000274C",
    ":world:": "\U0001F30D",
    ":rainbow:": "\U0001F308",
    ":idea:": "\U0001F4A1",
    ":pin:": "\U0001F4CC",
    ":construction:": "\U0001F6A7",
    ":rocket:": "\U0001F680",
    ":hourglass:": "\U000023F3",
    ":prohibited:": "\U0001F6AB",
    ":checked:": "\U00002705",
    ":smile:": "\U0001F642",
    ":sad:": "\U0001F612",
    ":star:": "\U00002B50",
    ":zzz:": "\U0001F4A4",
    ":mu:": "\U000003BC",
    ":pi:": "\U000003C0",
    ":sigma:": "\U000003C3",
    ":rightarrow:": "\U000027A1",
}


# terminal or notebook can do color print
def _has_colors(stream):
    try:
        import IPython
        return True
    except:
        pass

    if not hasattr(stream, "isatty"):
        return False
    if not stream.isatty():
        return False
    return True
_terminal_has_colors = _has_colors(sys.stdout)


def _is_sequence(arg):
    # Check if input is iterable.
    if hasattr(arg, "strip"):
        return False
    if hasattr(arg, "__getslice__"):
        return True
    if hasattr(arg, "__iter__"):
        return True
    return False


def getColor(rgb=None, hsv=None):
    """Deprecated. Use `get_color()`"""
    return get_color(rgb, hsv)


def get_color(rgb=None, hsv=None):
    """
    Convert a color or list of colors to (r,g,b) format from many different input formats.

    Set `hsv` to input as (hue, saturation, value).

    Example:
         - `RGB    = (255, 255, 255)` corresponds to white
         - `rgb    = (1,1,1)` is again white
         - `hex    = #FFFF00` is yellow
         - `string = 'white'`
         - `string = 'w'` is white nickname
         - `string = 'dr'` is darkred
         - `string = 'red4'` is a shade of red
         - `int    =  7` picks color nr. 7 in a predefined color list
         - `int    = -7` picks color nr. 7 in a different predefined list


    Examples:
        - [colorcubes.py](https://github.com/marcomusy/vedo/tree/master/examples/basic/colorcubes.py)

            ![](https://vedo.embl.es/images/basic/colorcubes.png)
    """
    # recursion, return a list if input is list of colors:
    if _is_sequence(rgb) and (len(rgb) > 3 or _is_sequence(rgb[0])):
        seqcol = []
        for sc in rgb:
            seqcol.append(get_color(sc))
        return seqcol

    # because they are most common:
    if isinstance(rgb, str):
        if rgb == "r":
            return (0.9960784313725, 0.11764705882352, 0.121568627450980)
        elif rgb == "g":
            return (0.0156862745098, 0.49803921568627, 0.062745098039215)
        elif rgb == "b":
            return (0.0588235294117, 0.0, 0.984313725490196)

    if str(rgb).isdigit():
        rgb = int(rgb)

    if hsv:
        c = hsv2rgb(hsv)
    else:
        c = rgb

    if _is_sequence(c):
        if c[0] <= 1 and c[1] <= 1 and c[2] <= 1:
            return c  # already rgb
        if len(c) == 3:
            return list(np.array(c) / 255.0)  # RGB
        return (c[0] / 255.0, c[1] / 255.0, c[2] / 255.0, c[3])  # RGBA

    elif isinstance(c, str):  # is string
        c = c.replace("grey", "gray").replace(" ", "")
        if 0 < len(c) < 3:  # single/double letter color
            if c.lower() in color_nicks:
                c = color_nicks[c.lower()]
            else:
                vedo.logger.warning(
                    f"Unknown color nickname {c}\nAvailable abbreviations: {color_nicks}"
                )
                return (0.5, 0.5, 0.5)

        if c.lower() in colors:  # matplotlib name color
            c = colors[c.lower()]
            # from now format is hex!

        if c.startswith("#"):  # hex to rgb
            h = c.lstrip("#")
            rgb255 = list(int(h[i : i + 2], 16) for i in (0, 2, 4))
            rgbh = np.array(rgb255) / 255.0
            if np.sum(rgbh) > 3:
                vedo.logger.error(f"in get_color(): Wrong hex color {c}")
                return (0.5, 0.5, 0.5)
            return tuple(rgbh)

        else:  # vtk name color
            namedColors = vtk.vtkNamedColors()
            rgba = [0, 0, 0, 0]
            namedColors.GetColor(c, rgba)
            return (rgba[0] / 255.0, rgba[1] / 255.0, rgba[2] / 255.0)

    elif isinstance(c, (int, float)):  # color number
        return palettes[vedo.settings.palette % len(palettes)][abs(int(c)) % 10]

    return (0.5, 0.5, 0.5)


def get_color_name(c):
    """Find the name of the closest color."""
    c = np.array(get_color(c))  # reformat to rgb
    mdist = 99.0
    kclosest = ""
    for key in colors:
        ci = np.array(get_color(key))
        d = np.linalg.norm(c - ci)
        if d < mdist:
            mdist = d
            kclosest = str(key)
    return kclosest


def hsv2rgb(hsv):
    """Convert HSV to RGB color."""
    ma = vtk.vtkMath()
    rgb = [0, 0, 0]
    ma.HSVToRGB(hsv, rgb)
    return rgb


def rgb2hsv(rgb):
    """Convert RGB to HSV color."""
    ma = vtk.vtkMath()
    hsv = [0, 0, 0]
    ma.RGBToHSV(get_color(rgb), hsv)
    return hsv


def rgb2hex(rgb):
    """Convert RGB to Hex color."""
    h = "#%02x%02x%02x" % (int(rgb[0] * 255), int(rgb[1] * 255), int(rgb[2] * 255))
    return h


def hex2rgb(hx):
    """Convert Hex to rgb color."""
    h = hx.lstrip("#")
    rgb255 = [int(h[i : i + 2], 16) for i in (0, 2, 4)]
    return (rgb255[0] / 255.0, rgb255[1] / 255.0, rgb255[2] / 255.0)


def colorMap(value, name="jet", vmin=None, vmax=None):
    """Deprecated, use `color_map()`"""
    print("Deprecated call to colorMap(), use color_map() instead")
    return color_map(value, name, vmin, vmax)


def color_map(value, name="jet", vmin=None, vmax=None):
    """
    Map a real value in range [vmin, vmax] to a (r,g,b) color scale.

    Return the (r,g,b) color, or a list of (r,g,b) colors.

    Arguments:
        value : *(float, list)*
            scalar value to transform into a color
        name : *(str, matplotlib.colors.LinearSegmentedColormap)*
            color map name

    Very frequently used color maps are:

        ![](https://user-images.githubusercontent.com/32848391/50738804-577e1680-11d8-11e9-929e-fca17a8ac6f3.jpg)

    A more complete color maps list:

        ![](https://matplotlib.org/1.2.1/_images/show_colormaps.png)

    .. note:: Can also directly use and customize a matplotlib color map

    Example:
        ```python
        from vedo import color_map
        import matplotlib.cm as cm
        print( color_map(0.2, cm.flag, 0, 1) )
        # (1.0, 0.809016994374948, 0.6173258487801733)
        ```

    Examples:
        - [plot_bars.py](https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_bars.py)

            <img src="https://vedo.embl.es/images/pyplot/plot_bars.png" width="400"/>

    """
    cut = _is_sequence(value)  # to speed up later

    if cut:
        values = np.asarray(value)
        if vmin is None:
            vmin = np.min(values)
        if vmax is None:
            vmax = np.max(values)
        values = np.clip(values, vmin, vmax)
        values = (values - vmin) / (vmax - vmin)
    else:
        if vmin is None:
            vedo.logger.warning("in color_map() you must specify vmin! Assume 0.")
            vmin = 0
        if vmax is None:
            vedo.logger.warning("in color_map() you must specify vmax! Assume 1.")
            vmax = 1
        values = [(value - vmin) / (vmax - vmin)]

    if _has_matplotlib:
        # matplotlib is available, use it! ###########################
        if isinstance(name, str):
            mp = _cm_mpl.get_cmap(name=name)
        else:
            mp = name  # assume matplotlib.colors.LinearSegmentedColormap
        result = mp(values)[:, [0, 1, 2]]

    else:
        # matplotlib not available ###################################
        invert = False
        if name.endswith("_r"):
            invert = True
            name = name.replace("_r", "")
        try:
            cmap = cmaps[name]
        except KeyError:
            vedo.logger.error(f"in color_map(), no color map with name {name} or {name}_r")
            vedo.logger.error(f"Available color maps are:\n{cmaps.keys()}")
            return np.array([0.5, 0.5, 0.5])

        result = []
        n = len(cmap) - 1
        for v in values:
            iv = int(v * n)
            if invert:
                iv = n - iv
            rgb = hex2rgb(cmap[iv])
            result.append(rgb)
        result = np.array(result)

    if cut:
        return result
    return result[0]


def build_palette(color1, color2, n, hsv=True):
    """
    Generate N colors starting from `color1` to `color2`
    by linear interpolation in HSV or RGB spaces.

    Arguments:
        N : (int)
            number of output colors.
        color1 : (color)
            first color.
        color2 : (color)
            second color.
        hsv : (bool)
            if `False`, interpolation is calculated in RGB space.

    Examples:
        - [mesh_custom.py](https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_custom.py)

            ![](https://vedo.embl.es/images/basic/mesh_custom.png)
    """
    if hsv:
        color1 = rgb2hsv(color1)
        color2 = rgb2hsv(color2)
    c1 = np.array(get_color(color1))
    c2 = np.array(get_color(color2))
    cols = []
    for f in np.linspace(0, 1, n, endpoint=True):
        c = c1 * (1 - f) + c2 * f
        if hsv:
            c = np.array(hsv2rgb(c))
        cols.append(c)
    return np.array(cols)


def buildLUT(
    colorlist,
    vmin=None,
    vmax=None,
    belowColor=None,
    aboveColor=None,
    nanColor=None,
    belowAlpha=1,
    aboveAlpha=1,
    nanAlpha=1,
    interpolate=False,
):
    """Deprecated. Please use `build_lut()`"""
    print("Deprecated call to buildLUT(). Please use build_lut() instead")
    return build_lut(
        colorlist,
        vmin,
        vmax,
        belowColor,
        aboveColor,
        nanColor,
        belowAlpha,
        aboveAlpha,
        nanAlpha,
        interpolate,
    )


def build_lut(
    colorlist,
    vmin=None,
    vmax=None,
    below_color=None,
    above_color=None,
    nan_color=None,
    below_alpha=1,
    above_alpha=1,
    nan_alpha=1,
    interpolate=False,
):
    """
    Generate colors in a lookup table (LUT).

    Return the `vtkLookupTable` object. This can be fed into `cmap()` method.

    Arguments:
        colorlist : (list)
            a list in the form `[(scalar1, [r,g,b]), (scalar2, 'blue'), ...]`.
        vmin : (float)
            specify minimum value of scalar range
        vmax : (float)
            specify maximum value of scalar range
        below_color : (color)
            color for scalars below the minimum in range
        below_alpha : (float)
            opacity for scalars below the minimum in range
        above_color : (color)
            color for scalars above the maximum in range
        above_alpha : (float)
            alpha for scalars above the maximum in range
        nan_color : (color)
            color for invalid (nan) scalars
        nan_alpha : (float)
            alpha for invalid (nan) scalars
        interpolate : (bool)
            interpolate or not intermediate scalars

    Examples:
        - [mesh_lut.py](https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_lut.py)

            ![](https://vedo.embl.es/images/basic/mesh_lut.png)
    """
    ctf = vtk.vtkColorTransferFunction()
    ctf.SetColorSpaceToRGB()
    ctf.SetScaleToLinear()
    alpha_x, alpha_vals = [], []
    for sc in colorlist:
        if len(sc) >= 3:
            scalar, col, alf = sc[:3]
        else:
            alf = 1
            scalar, col = sc
        r, g, b = get_color(col)
        ctf.AddRGBPoint(scalar, r, g, b)
        alpha_x.append(scalar)
        alpha_vals.append(alf)

    lut = vtk.vtkLookupTable()
    lut.SetNumberOfTableValues(256)

    x0, x1 = ctf.GetRange()  # range of the introduced values
    if vmin is not None:
        x0 = vmin
    if vmax is not None:
        x1 = vmax
    ctf.SetRange(x0, x1)
    lut.SetRange(x0, x1)

    if below_color is not None:
        lut.SetBelowRangeColor(list(get_color(below_color)) + [below_alpha])
        lut.SetUseBelowRangeColor(True)
    if above_color is not None:
        lut.SetAboveRangeColor(list(get_color(above_color)) + [above_alpha])
        lut.SetUseAboveRangeColor(True)
    if nan_color is not None:
        lut.SetNanColor(list(get_color(nan_color)) + [nan_alpha])

    rgba = (1, 1, 1, 1)
    for i in range(256):
        p = i / 255
        x = (1 - p) * x0 + p * x1
        if interpolate:
            alf = np.interp(x, alpha_x, alpha_vals)
            rgba = list(ctf.GetColor(x)) + [alf]
        else:
            for c in colorlist:
                if x <= c[0]:
                    if len(c) == 3:
                        alf = c[2]
                    else:
                        alf = 1
                    rgba = list(get_color(c[1])) + [alf]
                    break
        lut.SetTableValue(i, rgba)

    lut.Build()
    return lut


#########################################################################
def printc(
    *strings,
    c=None,
    bc=None,
    bold=True,
    italic=False,
    blink=False,
    underline=False,
    strike=False,
    dim=False,
    invert=False,
    box="",
    end="\n",
    flush=True,
):
    """
    Print to terminal in color (any color!).

    Arguments:
        c : (color)
            foreground color name or (r,g,b)
        bc : (color)
            background color name or (r,g,b)
        bold : (bool)
            boldface [True]
        italic : (bool)
            italic [False]
        blink : (bool)
            blinking text [False]
        underline : (bool)
            underline text [False]
        strike : (bool)
            strike through text [False]
        dim : (bool)
            make text look dimmer [False]
        invert : (bool)
            invert background and forward colors [False]
        box : (bool)
            print a box with specified text character ['']
        flush : (bool)
            flush buffer after printing [True]
        end : (str)
            the end character to be printed [newline]

    Example:
        ```python
        from vedo.colors import printc
        printc('anything', c='tomato', bold=False, end=' ')
        printc('anything', 455.5, c='lightblue')
        printc(299792.48, c=4)
        ```

    Examples:
        - [printc.py](https://github.com/marcomusy/vedo/tree/master/examples/other/printc.py)

        ![](https://user-images.githubusercontent.com/32848391/50739010-2bfc2b80-11da-11e9-94de-011e50a86e61.jpg)
    """

    if not vedo.settings.enable_print_color or not _terminal_has_colors:
        print(*strings, end=end, flush=flush)
        return

    try:  # -------------------------------------------------------------

        txt = str()
        ns = len(strings) - 1
        separator = " "
        offset = 0
        for i, s in enumerate(strings):
            if i == ns:
                separator = ""
            if ":" in repr(s):
                for k in emoji:
                    if k in str(s):
                        s = s.replace(k, emoji[k])
                        offset += 1
                for k, rp in vedo.shapes._reps:  # check symbols in shapes._reps
                    if k in str(s):
                        s = s.replace(k, rp)
                        offset += 1

            txt += str(s) + separator

        special, cseq = "", ""
        oneletter_colors = {
            "k": "\u001b[30m",  # because these are supported by most terminals
            "r": "\u001b[31m",
            "g": "\u001b[32m",
            "y": "\u001b[33m",
            "b": "\u001b[34m",
            "m": "\u001b[35m",
            "c": "\u001b[36m",
            "w": "\u001b[37m",
        }

        if c is not None:
            if c is True:
                c = "g"
            elif c is False:
                c = "r"

            if isinstance(c, str) and c in oneletter_colors:
                cseq += oneletter_colors[c]
            else:
                r, g, b = get_color(c)  # not all terms support this syntax
                cseq += f"\x1b[38;2;{int(r*255)};{int(g*255)};{int(b*255)}m"

        if bc:
            if bc in oneletter_colors:
                cseq += oneletter_colors[bc]
            else:
                r, g, b = get_color(bc)
                cseq += f"\x1b[48;2;{int(r*255)};{int(g*255)};{int(b*255)}m"

        if box is True:
            box = "-"
        if underline and not box:
            special += "\x1b[4m"
        if strike and not box:
            special += "\x1b[9m"
        if dim:
            special += "\x1b[2m"
        if invert:
            special += "\x1b[7m"
        if bold:
            special += "\x1b[1m"
        if italic:
            special += "\x1b[3m"
        if blink:
            special += "\x1b[5m"

        if box and "\n" not in txt:
            box = box[0]
            boxv = box
            if box in ["_", "=", "-", "+", "~"]:
                boxv = "|"

            if box in ("_", "."):
                outtxt = special + cseq + " " + box * (len(txt) + offset + 2) + " \n"
                outtxt += boxv + " " * (len(txt) + 2) + boxv + "\n"
            else:
                outtxt = special + cseq + box * (len(txt) + offset + 4) + "\n"

            outtxt += boxv + " " + txt + " " + boxv + "\n"

            if box == "_":
                outtxt += "|" + box * (len(txt) + offset + 2) + "|" + reset + end
            else:
                outtxt += box * (len(txt) + offset + 4) + reset + end

            sys.stdout.write(outtxt)

        else:

            out = special + cseq + txt + reset
            sys.stdout.write(out + end)

    except:  # ------------------------------------------------------------- fallback

        try:
            print(*strings, end=end)
        except UnicodeEncodeError as e:
            print(e, end=end)
            pass

    if flush:
        sys.stdout.flush()


def printd(*strings, q=False):
    """
    Print debug information about the environment where the printd() is called.
    Local variables are printed out with their current values.

    Use `q` to quit (exit) python session after the printd call.
    """
    from inspect import currentframe, getframeinfo

    cf = currentframe().f_back
    cfi = getframeinfo(cf)

    fname = os.path.basename(getframeinfo(cf).filename)
    print("\x1b[7m\x1b[3m\x1b[37m" + fname + " line:\x1b[1m" + str(cfi.lineno) + reset, end="")
    print("\x1b[3m\x1b[37m\x1b[2m", "\U00002501" * 30, time.ctime(), reset)
    if strings:
        print("    \x1b[37m\x1b[1mMessage : ", *strings)
    print("    \x1b[37m\x1b[1mFunction:\x1b[0m\x1b[37m " + str(cfi.function))
    print("    \x1b[1mLocals  :" + reset)
    for loc in cf.f_locals.keys():
        obj = cf.f_locals[loc]
        var = repr(obj)
        if 'module ' in var: continue
        if 'function ' in var: continue
        if 'class ' in var: continue
        if loc.startswith('_'): continue
        if hasattr(obj, 'name'):
            if not obj.name:
                oname = str(type(obj))
            else:
                oname = obj.name
            var = oname + ", at " + vedo.utils.precision(obj.GetPosition(), 3)

        var = var.replace("vtkmodules.", "")
        print("      \x1b[37m", loc, "\t\t=", var[:60].replace("\n", ""), reset)
        if vedo.utils.is_sequence(obj) and len(obj) > 4:
            print('           \x1b[37m\x1b[2m\x1b[3m len:', len(obj),
                  ' min:', vedo.utils.precision(min(obj), 4),
                  ' max:', vedo.utils.precision(max(obj), 4),
                  reset)

    if q:
        print(f"    \x1b[1m\x1b[37mExiting python now (q={bool(q)}).\x1b[0m\x1b[37m")
        sys.exit(0)
    sys.stdout.flush()



from vedo import *

vol = Volume(dataurl+"embryo.slc").cmap('nipy_spectral')

vsl = VolumeSlice(vol) # reuse the same underlying data as in vol

# use colorize("bw") to have black and white color scale
# no argument will grab the existing cmap in vol (or use build_lut())
vsl.colorize().lighting(window=100, level=25)

usage = Text2D(
    f"Image-style interactor:\n"
    f"SHIFT+Left click   :rightarrow rotate camera for oblique slicing\n"
    f"SHIFT+Middle click :rightarrow slice perpendicularly through image\n"
    f"Left click & drag  :rightarrow modify luminosity and contrast\n"
    f"X                  :rightarrow Reset to sagittal view\n"
    f"Y                  :rightarrow Reset to coronal view\n"
    f"Z                  :rightarrow Reset to axial view\n"
    f"R                  :rightarrow Reset the Window/Levels",
    font="Calco", pos="bottom-left", s=0.9, bg='yellow', alpha=0.25
)

custom_shape = [ # define here the 2 rendering rectangle spaces
    dict(bottomleft=(0.0,0.0), topright=(1,1), bg='k9'), # the full window
    dict(bottomleft=(0.7,0.7), topright=(1,1), bg='k8', bg2='lb'),
]

show([ (vsl,usage,"VolumeSlice example"), (vol,"Volume") ],
     shape=custom_shape,
     mode="image", bg='k9', zoom=1.2, axes=11, interactive=1).close()



from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk, vuetify

import vedo

cone = vedo.Cone()
plt = vedo.Plotter()
plt += [cone, vedo.Axes(cone).unpack(transformed=True)]

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------
server = get_server()
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Functions
# -----------------------------------------------------------------------------
@state.change("resolution")
def update_resolution(resolution, **kwargs):
    cone.color(resolution)
    ctrl.view_update()

def reset_resolution():
    cone.color("red5")
    ctrl.view_update()

# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------
with SinglePageLayout(server) as layout:
    layout.title.set_text("Use slider to change color")

    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            plt.reset_camera()
            view = vtk.VtkLocalView(plt.window)
            ctrl.view_update = view.update
            ctrl.view_reset_camera = view.reset_camera

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            v_model=("resolution", "blue5"),
            min=3,
            max=60,
            step=1,
            hide_details=True,
            dense=True,
            style="max-width: 300px",
        )
        with vuetify.VBtn(icon=True, click=reset_resolution):
            vuetify.VIcon("mdi-restore")

        vuetify.VDivider(vertical=True, classes="mx-2")

        vuetify.VSwitch(
            v_model="$vuetify.theme.dark",
            hide_details=True,
            dense=True,
        )
        with vuetify.VBtn(icon=True, click=ctrl.view_reset_camera):
            vuetify.VIcon("mdi-crop-free")

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------
server.start()



"""Align 2 shapes:
the red line to the yellow surface"""
from vedo import *

# Load two mesh objects, a limb and a rim, and color them gold and red
limb = Mesh(dataurl + "270.vtk").c("gold")
rim = Mesh(dataurl + "270_rim.vtk").c("red5").lw(4)

# Make a clone copy of the rim and align it to the limb
# Using rigid=True does not allow scaling
rim2 = rim.clone().align_to(limb, rigid=True).c("green5").lw(5)

# Calculate the average squared distance between the aligned rim and the limb
d = 0
for p in rim2.points():
    cpt = limb.closest_point(p)
    d += mag2(p - cpt)  # square of residual distance
average_squared_distance = d / rim2.npoints

# Print the average squared distance between the aligned rim and the limb
printc("Average squared distance =", average_squared_distance, c="g")

show(limb, rim, rim2, __doc__, axes=1).close()



"""
Takes 2 shapes, source and target, and morphs source on target
this is obtained by fitting 18 parameters of a non linear,
quadratic, transformation defined in transform()
The fitting minimizes the distance to the target surface
using algorithms available in the scipy.optimize package.
"""
from vedo import dataurl, vector, mag2, mag
from vedo import Plotter, Sphere, Point, Text3D, Arrows, Mesh
import scipy.optimize as opt

print(__doc__)

plt = Plotter(shape=[1, 3], interactive=0, axes=1)


class Morpher:
    def __init__(self):
        self.source = None
        self.target = None
        self.bound = 0.1
        self.method = "SLSQP"  # 'SLSQP', 'L-BFGS-B', 'TNC' ...
        self.tolerance = 0.0001
        self.subsample = 200  # pick only subsample pts
        self.allow_scaling = False
        self.params = []

        self.msource = None
        self.s_size = ([0, 0, 0], 1)  # ave position and ave size
        self.fitResult = None
        self.chi2 = 1.0e10

    # -------------------------------------------------------- fit function
    def transform(self, p):
        a1, a2, a3, a4, a5, a6, b1, b2, b3, b4, b5, b6, c1, c2, c3, c4, c5, c6, s = self.params
        pos, sz = self.s_size[0], self.s_size[1]
        x, y, z = (p - pos) / sz * s  # bring to origin, norm and scale
        xx, yy, zz, xy, yz, xz = x * x, y * y, z * z, x * y, y * z, x * z
        xp = x + 2 * a1 * xy + a4 * xx + 2 * a2 * yz + a5 * yy + 2 * a3 * xz + a6 * zz
        yp = +2 * b1 * xy + b4 * xx + y + 2 * b2 * yz + b5 * yy + 2 * b3 * xz + b6 * zz
        zp = +2 * c1 * xy + c4 * xx + 2 * c2 * yz + c5 * yy + z + 2 * c3 * xz + c6 * zz
        p2 = vector(xp, yp, zp)
        p2 = (p2 * sz) + pos  # take back to original size and position
        return p2

    def _func(self, pars):
        self.params = pars

        #calculate chi2
        d2sum, n = 0.0, self.source.npoints
        srcpts = self.source.points()
        rng = range(0, n, int(n / self.subsample))
        for i in rng:
            p1 = srcpts[i]
            p2 = self.transform(p1)
            tp = self.target.closest_point(p2)
            d2sum += mag2(p2 - tp)
        d2sum /= len(rng)

        if d2sum < self.chi2:
            if d2sum < self.chi2 * 0.99:
                print("Emin ->", d2sum)
            self.chi2 = d2sum
        return d2sum

    # ------------------------------------------------------- Fit
    def morph(self):
        def avesize(pts):  # helper fnc
            s, amean = 0, vector(0, 0, 0)
            for p in pts:
                amean = amean + p
            amean /= len(pts)
            for p in pts:
                s += mag(p - amean)
            return amean, s / len(pts)

        print("\n..minimizing with " + self.method)
        self.msource = self.source.clone()

        self.s_size = avesize(self.source.points())
        bnds = [(-self.bound, self.bound)] * 18
        x0 = [0.0] * 18  # initial guess
        x0 += [1.0]  # the optional scale
        if self.allow_scaling:
            bnds += [(1.0 - self.bound, 1.0 + self.bound)]
        else:
            bnds += [(1.0, 1.0)]  # fix scale to 1
        res = opt.minimize(self._func, x0,
                           bounds=bnds, method=self.method, tol=self.tolerance)
        # recalc for all pts:
        self.subsample = self.source.npoints
        self._func(res["x"])
        print("\nFinal fit score", res["fun"])
        self.fitResult = res

    # ------------------------------------------------------- Visualization
    def draw_shapes(self):

        pos, sz = self.s_size[0], self.s_size[1]

        sphere0 = Sphere(pos, c="gray", r=sz, res=10, quads=True).wireframe()

        newpts = []
        for p in self.msource.points():
            newp = self.transform(p)
            newpts.append(newp)
        self.msource.points(newpts)

        arrs = []
        for p in sphere0.points():
            newp = self.transform(p)
            arrs.append([p, newp])
        hair = Arrows(arrs, s=0.3, c='jet')

        zero = Point(pos, c="black")
        x1, x2, y1, y2, z1, z2 = self.target.bounds()
        tpos = [x1, y2, z1]
        text1 = Text3D("source vs target", tpos, s=sz / 10, c="dg")
        text2 = Text3D("morphed vs target", tpos, s=sz / 10, c="dg")
        text3 = Text3D("deformation", tpos, s=sz / 10, c="dr")

        plt.at(2).show(sphere0, zero, text3, hair)
        plt.at(1).show(self.msource, self.target, text2)
        plt.at(0).show(self.source, self.target, text1, zoom=1.2, interactive=True)
        plt.close()


#################################
if __name__ == "__main__":

    mr = Morpher()
    mr.source = Mesh(dataurl+"270.vtk").color("g").alpha(0.4)
    mr.target = Mesh(dataurl+"290.vtk").color("b").alpha(0.3)
    mr.target.wireframe()
    mr.allow_scaling = True
    mr.bound = 0.4  # limits the parameter value

    mr.morph()

    print("Result of parameter fit:\n", mr.params)

    # now mr.msource contains the modified/morphed source.
    mr.draw_shapes()



"""Remove points and cells from a mesh
which are closest to a specified point."""
from vedo import *

# Enable depth peeling for the scene
settings.use_depth_peeling = True

# Load the apple mesh from a url, set the colors and line width
msh = Mesh(dataurl+'apple.ply')
msh.c('lightgreen').bc('tomato').lw(0.1)

# Set a point and a radius to find the closest points in the mesh to it
pt = [1, 0.5, 1]
R = 1.2
ids = msh.closest_point(pt, radius=R, return_point_id=True)

# Remove the cells from the mesh by their ids
# and clean the mesh by removing orphaned vertices not associated to any cell
printc('#points before:', msh.npoints, c='g')
msh.delete_cells_by_point_index(ids)
msh.clean()
printc('#points after :', msh.npoints, c='g')

# Create a sphere object with the given point and radius, and set transparency
sph = Sphere(pt, r=R, alpha=0.1)

# Show the point, the sphere, the modified mesh, the script docstring and axes
# then close the window
show(Point(pt), sph, msh, __doc__, axes=1).close()



"""Manual capping of a mesh"""
from vedo import *

def capping(amsh, bias=0, invert=False, res=50):

    bn =  amsh.boundaries().join(reset=True)

    pln = fit_plane(bn)
    cp = [pln.closest_point(p) for p in bn.points()]
    pts = Points(cp)
    pts.top = pln.normal

    if invert is None:
        cutm = amsh.clone().cut_with_plane(origin=pln.center, normal=pln.normal)
        invert = cutm.npoints > amsh.npoints

    pts2 = pts.clone().orientation([0,0,1]).project_on_plane('z')
    msh2 = pts2.generate_mesh(invert=invert, mesh_resolution=res)

    source = pts2.points().tolist()
    target = bn.points().tolist()
    printc(f"..warping {len(source)} points")
    msh3 = msh2.clone().warp(source, target, mode='3d')

    if not invert:
        bias *= -1
        msh3.reverse()

    if bias:
        newpts = []
        for p in msh3.points():
            q = bn.closest_point(p)
            d = mag(p-q)
            newpt = p + d * pln.normal * bias
            newpts.append(newpt)
        msh3.points(newpts)
    return msh3


msh = Mesh(dataurl+"260_flank.vtp").c('orange5').bc('purple7').lw(0.1)

# mcap = msh.cap()  # automatic
mcap = capping(msh, invert=True)

merged_msh = merge(msh, mcap).clean().smooth()
merged_msh.subsample(0.0001).wireframe(False)  # merge duplicate points
printc("merged_msh is closed:", merged_msh.is_closed())

show([[msh, __doc__],
      [merged_msh, merged_msh.boundaries()]],
      N=2, axes=1, elevation=-40,
).close()




"""Read structured grid data and show
the associated vector and scalar fields"""
from vedo import *

settings.use_depth_peeling = True

g = load(dataurl+'structgrid.vts')

coords = g.points()

# g.print() gives the list of point and cell data contained in g
vects  = g.pointdata['Momentum']/600
print('numpy array shapes are:', coords.shape, vects.shape)

# build arrows from starting points to endpoints, with colormap
arrows = Arrows(coords-vects, coords+vects, c='hot_r')

g.cmap('jet', input_array='Density').linewidth(0.1).alpha(0.3)

show(g, arrows, __doc__, axes=7, viewup='z').close()



"""Delaunay 2D meshing
with point loops defining holes"""
from vedo import *

# Generate a grid and add gaussian noise to it
# then extract the points from the grid and store them in the variable gp
gp = Grid().add_gaussian_noise([0.5,0.5,0]).points()

# Define two internal holes using point ids
ids = [[24,35,36,37,26,15,14,25], [84,95,96,85]]

# Create a point cloud object using the extracted points gp
pts = Points(gp, r=6).c('blue3')

# Use the Delaunay triangulation algorithm to create a 2D mesh 
# from the points in gp, with the given boundary ids
dly = delaunay2d(gp, mode='xy', boundaries=ids).c('w').lc('o').lw(1)

# Create labels for the point ids and set their z to 0.01
labels = pts.labels('id').z(0.01)

show(pts, labels, dly, __doc__, bg="Mint").close()



"""Plot a histogram of the distance of each point 
of a sphere to the oceans mesh. The distance is used to
threshold the sphere and create continents."""
from vedo import *
from vedo.pyplot import histogram

# Download the oceans mesh
oceans = Mesh(dataurl + "oceans.vtk").c("blue9")
size = oceans.average_size()

# Create a sphere and compute the distance to the oceans mesh
sphere = IcoSphere(subdivisions=5).scale(size*1.01)
dists = sphere.distance_to(oceans)

# Create a histogram of the distance
histo = histogram(dists, logscale=True, c="gist_earth")
histo+= Arrow2D([200,1], [200,0]).z(1).c("red5")

# Threshold the sphere to create continents
continents = sphere.threshold("Distance", above=20.0)
continents.cmap("gist_earth").linewidth(1)

show(oceans, continents, histo.as2d(), __doc__)




"""Align to bounding boxes. Force the Mesh into the empty box."""
from vedo import *

# Load a mesh and color it silver
msh1 = Mesh(dataurl + "cessna.vtk").color("silver")

# Create axes for the original mesh
axes1 = Axes(msh1)

# Create a wireframe cube at a specified position
cube = Cube().pos(2, 1, 0).wireframe()

# Clone the mesh and align it to the bounding box of the cube
msh2 = msh1.clone().align_to_bounding_box(cube)

# Create axes for the aligned mesh
axes2 = Axes(msh2)

# Set up a Plotter object with 2 subrenderers
plt = Plotter(N=2)

# Show the original mesh, axes, and cube in the left renderer with the script description
plt.at(0).show(msh1, axes1, cube, __doc__)

# Show the aligned mesh and axes in the right renderer, viewing from the top
plt.at(1).show(msh2, axes2, viewup='z')

# Enable interaction and close the plotter when done
plt.interactive().close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# See more examples at:
# https://github.com/Kitware/trame-tutorial

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk, vuetify

import vedo

cone = vedo.Cone()

plt = vedo.Plotter()
plt += cone

# -----------------------------------------------------------------------------
# Trame
# -----------------------------------------------------------------------------
server = get_server()

with SinglePageLayout(server) as layout:
    layout.title.set_text("Hello trame")

    with layout.content:

        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            plt.reset_camera()
            view = vtk.VtkLocalView(plt.window)

server.start()



"""Fit y=ax+b and compute error bands"""
from vedo import Text2D, DashedLine, show
from vedo.pyplot import plot, fit
import numpy as np
# np.random.seed(0)

# Generate some noisy data points along a line
a, b = (np.random.rand(2)-0.5)*10 # choose a and b

x = np.linspace(0, 15, 25)
y = a*x + b
noise = np.random.randn(len(x)) * 5 # create gaussian noise

# Plot the points and add the "true" line without noise
fig = plot(x, y+noise, '*k', title=__doc__, label='data')
fig += DashedLine(x, y, c='red5')

# Fit points and evaluate, with a bootstrap and Monte-Carlo technique,
# the correct error coeffs and error bands. Return a Line object:
pfit = fit(
    [x, y+noise],
    deg=1,          # degree of the polynomial
    niter=500,      # nr. of MC iterations to compute error bands
    nstd=2,         # nr. of std deviations to display
)

fig += [pfit, pfit.error_band, *pfit.error_lines] # add these objects to fig

msg = f"Generated a, b  : {np.array([a,b])}"\
      f"\nFitted    a, b  : {pfit.coefficients}"\
      f"\nerrors on a, b  : {pfit.coefficient_errors}"\
      f"\nave point spread: :sigma :approx {pfit.data_sigma:.3f} in y units"
msg = Text2D(msg, font='VictorMono', pos='bottom-left', c='red3')

fig.add_label("y = a:dotx+b", c='k', marker='-', mc='red4')
fig.add_label("ground truth", c='k', marker='--', mc='red5')
fig.add_label("1:sigma error band", c='k', marker='s', mc='grey')
fig.add_legend(pos=[1.07,1], vspace=2)

show(fig, msg, zoom=1.1).close()



"""Draw the ellipsoid that contains 50% of a cloud of Points,
then check how many points are inside the surface"""
#
# NB: check out pcaEllipse() method for 2D problems
#
from vedo import *

settings.use_depth_peeling = True

pts = Points(np.random.randn(10000, 3)*[3,2,1] + [50,60,70])

elli = pca_ellipsoid(pts, pvalue=0.50)

elli.inside_points(pts, return_ids=True)

ids  = elli.inside_points(pts, return_ids=True)
pts.print()  # a new "IsInside" array now exists in pts
pin = pts.points()[ids]
print("inside  points #", len(pin))

# Create an inverted mask instead of calling insidePoints(invert=True)
mask = np.ones(pts.npoints, dtype=bool)
mask[ids] = False
pout = pts.points()[mask]
print("outside  points #", len(pout))

# Extra info can be retrieved with:
print("axis 1 size:", elli.va)
print("axis 2 size:", elli.vb)
print("axis 3 size:", elli.vc)
print("axis 1 direction:", elli.axis1)
print("axis 2 direction:", elli.axis2)
print("axis 3 direction:", elli.axis3)
print("asphericity:", elli.asphericity(), '+-', elli.asphericity_error())

a1 = Arrow(elli.center, elli.center + elli.axis1)
a2 = Arrow(elli.center, elli.center + elli.axis2)
a3 = Arrow(elli.center, elli.center + elli.axis3)

show(elli,
     a1, a2, a3,
     Points(pin).c("green4"),
     Points(pout).c("red5").alpha(0.2),
     __doc__,
     axes=1,
).close()



"""Elliptic Fourier Descriptors
parametrizing a closed contour (in red)"""
import vedo
import pyefd

shapes = vedo.load(vedo.dataurl+'timecourse1d.npy')

sh = shapes[55]
sr = vedo.Line(sh).mirror('x').reverse()
sm = vedo.merge(sh, sr).c('red5').lw(3)
pts = sm.points()[:,(0,1)]

rlines = []
for order in range(5,30, 5):
    coeffs = pyefd.elliptic_fourier_descriptors(pts, order=order, normalize=False)
    a0, c0 = pyefd.calculate_dc_coefficients(pts)
    rpts = pyefd.reconstruct_contour(coeffs, locus=(a0,c0), num_points=400)
    color = vedo.color_map(order, "Blues", 5,30)
    rline = vedo.Line(rpts).lw(3).c(color)
    rlines.append(rline)

sm.z(0.1) # move it on top so it's visible
vedo.show(sm, *rlines, __doc__, axes=1, bg='k', size=(1190, 630), zoom=1.8).close()



import sys
from PyQt5 import Qt
from vtk.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor
from vedo import Plotter, Picture, Text2D, printc

class MainWindow(Qt.QMainWindow):

    def __init__(self, parent=None):

        Qt.QMainWindow.__init__(self, parent)
        self.frame = Qt.QFrame()
        self.layout = Qt.QVBoxLayout()
        self.vtkWidget = QVTKRenderWindowInteractor(self.frame)

        # Create vedo renderer and add objects and callbacks
        self.plt = Plotter(qt_widget=self.vtkWidget)
        self.cbid = self.plt.add_callback("key press", self.onKeypress)
        self.imgActor = Picture("https://icatcare.org/app/uploads/2018/07/Helping-your-new-cat-or-kitten-settle-in-1.png")
        self.text2d = Text2D("Use slider to change contrast")

        self.slider = Qt.QSlider(1)
        self.slider.valueChanged.connect(self.onSlider)
        self.layout.addWidget(self.vtkWidget)
        self.layout.addWidget(self.slider)

        self.frame.setLayout(self.layout)
        self.setCentralWidget(self.frame)
        self.plt.show(self.imgActor, self.text2d, mode='image') # build the vedo rendering
        self.show()                                            # show the Qt Window


    def onSlider(self, value):
        self.imgActor.window(value*10) # change image contrast
        self.text2d.text(f"window level is now: {value*10}")
        self.plt.render()

    def onKeypress(self, evt):
        printc("You have pressed key:", evt.keypress, c='b')
        if evt.keypress=='q':
            self.plt.close()
            self.vtkWidget.close()
            exit()

    def onClose(self):
        self.vtkWidget.close()

if __name__ == "__main__":
    app = Qt.QApplication(sys.argv)
    window = MainWindow()
    app.aboutToQuit.connect(window.onClose)
    app.exec_()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
##### To generate documentation
# cd ~/Projects/vedo/docs/pdoc
# ./build_html.py
###############################
"""
.. include:: ../docs/documentation.md
"""

######################################################################## imports
import os
import sys
import logging
import numpy as np
from numpy import sin, cos, sqrt, exp, log, dot, cross  # just because handy

from vtkmodules.vtkCommonCore import vtkVersion

#################################################
from vedo.version import _version as __version__

from vedo.settings import Settings
settings = Settings(level=0)

from vedo.colors import *
from vedo.utils import *
from vedo.base import *
from vedo.shapes import *
from vedo.file_io import *
from vedo.ugrid import *
from vedo.assembly import *
from vedo.pointcloud import *
from vedo.mesh import *
from vedo.picture import *
from vedo.volume import *
from vedo.tetmesh import *
from vedo.addons import *
from vedo.plotter import *

from vedo import applications
from vedo import interactor_modes

try:
    import platform
    sys_platform = platform.system()
except (ModuleNotFoundError, AttributeError) as e:
    sys_platform = ""

######################################################################### GLOBALS
__author__     = "Marco Musy"
__license__    = "MIT"
__maintainer__ = "M. Musy"
__email__      = "marco.musy@embl.es"
__website__    = "https://github.com/marcomusy/vedo"


##########################################################################
vtk_version = (
    int(vtkVersion().GetVTKMajorVersion()),
    int(vtkVersion().GetVTKMinorVersion()),
    int(vtkVersion().GetVTKBuildVersion()),
)

installdir = os.path.dirname(__file__)
dataurl = "https://vedo.embl.es/examples/data/"

plotter_instance = None
notebook_plotter = None
notebook_backend = None

## fonts
fonts_path = os.path.join(installdir, "fonts/")

# Note:
# a fatal error occurs when compiling to exe,
# developer needs to copy the fonts folder to the same location as the exe file
# to solve this problem
if not os.path.exists(fonts_path):
    fonts_path = "fonts/"

fonts = [_f.split(".")[0] for _f in os.listdir(fonts_path) if '.npz' not in _f]
fonts = list(sorted(fonts))

# pyplot module to remember last figure format
last_figure = None


######################################################################### LOGGING
class _LoggingCustomFormatter(logging.Formatter):

    logformat = "[vedo.%(filename)s] %(levelname)s: %(message)s"

    white = "\x1b[1m"
    grey = "\x1b[2m\x1b[1m\x1b[38;20m"
    yellow = "\x1b[1m\x1b[33;20m"
    red = "\x1b[1m\x1b[31;20m"
    inv_red = "\x1b[7m\x1b[1m\x1b[31;1m"
    reset = "\x1b[0m"

    FORMATS = {
        logging.DEBUG: grey  + logformat + reset,
        logging.INFO: white + logformat + reset,
        logging.WARNING: yellow + logformat + reset,
        logging.ERROR: red + logformat + reset,
        logging.CRITICAL: inv_red + logformat + reset,
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record).replace(".py", "")

logger = logging.getLogger("vedo")

_chsh = logging.StreamHandler()
_chsh.flush = sys.stdout.flush
_chsh.setLevel(logging.DEBUG)
_chsh.setFormatter(_LoggingCustomFormatter())
logger.addHandler(_chsh)
logger.setLevel(logging.INFO)

################################################# silence annoying messages
# import warnings
# warnings.simplefilter(action="ignore", category=FutureWarning)
# try:
#     np.warnings.filterwarnings('ignore', category=np.VisibleDeprecationWarning)
# except AttributeError:
#     pass





"""Superpose plots in different styles"""
from vedo.pyplot import plot
from vedo import np, settings

settings.default_font = 'Theemim'
settings.remember_last_figure_format = True

x = np.linspace(0, 10, num=21)
y = 3 * np.sin(x)

################# first plot
fig = plot(
    x, y,
    "*r-",           # markers: *,o,p,h,D,d,v,^,s,x,a
	title=__doc__,
    xtitle="t variable (:mus)",
    ytitle="y(x) = :pmK_i :dot:sqrtsin^2 t",
    aspect=16/9,     # aspect ratio x/y of plot
    xlim=(-1, 14),   # specify x range
    axes=dict(text_scale=1.2),
    label="3 :dot sin(x)",
)

################# plot on top of fig
fig += plot(
    x + np.pi, y,
    "sb--",
    like=fig,        # format like fig
    splined=True,    # continuous spline through points
    lw=3,            # line width
    label="3 :dot sin(x - :pi)",
)

################## plot again on top of fig
fig += plot(x, y/5, "g", label="3/5 :dot sin(x)")

################## plot again on top of fig
fig += plot(x, y/5-1, "purple5 -", label="3/5 :dot sin(x) - 1")

################## Show! ##################
fig.add_legend(pos=[0.95,1])
fig.show(size=(1400,900), zoom='tight').close()



"""Quantum tunneling using 4th order Runge-Kutta method"""
import numpy as np
from vedo import Plotter, Line

N = 300      # number of points
dt = 0.004   # time step
x0 = 5       # peak initial position
s0 = 0.75    # uncertainty on particle position
k0 = 10      # initial momentum of the wave packet
Vmax = 0.2   # height of the barrier (try 0 for particle in empty box)
size = 20.0  # x span [0, size]


def f(psi):
    nabla2psi = np.zeros(N+2, dtype=complex)
    dx2 = ((x[-1] - x[0]) / (N+2))**2 * 400  # dx**2 step, scaled
    nabla2psi[1 : N+1] = (psi[0:N] + psi[2 : N+2] - 2 * psi[1 : N+1]) / dx2
    return 1j * (nabla2psi - V * psi)  # this is the RHS of Schroedinger equation


def d_dt(psi):  # find Psi(t+dt)-Psi(t) /dt with 4th order Runge-Kutta method
    k1 = f(psi)
    k2 = f(psi + dt / 2 * k1)
    k3 = f(psi + dt / 2 * k2)
    k4 = f(psi + dt * k3)
    return (k1 + 2 * k2 + 2 * k3 + k4) / 6


x = np.linspace(0, size, N+2)  # we will need 2 extra points for the box wall
V = Vmax * (np.abs(x-11) < 0.5) - 0.01  # simple square barrier potential

Psi = np.sqrt(1/s0) * np.exp(-1/2 * ((x-x0)/s0)**2 + 1j * x * k0)  # wave packet
zeros = np.zeros_like(x)

plt = Plotter(interactive=False, size=(1000,500))

barrier = Line(np.c_[x, V * 15], c="red3", lw=3)
wpacket = Line(np.c_[x, zeros], c='blue4', lw=2)
plt.show(barrier, wpacket, __doc__, zoom=2)

for j in range(150):
    for i in range(500):
        Psi += d_dt(Psi) * dt                # integrate for a while
    amp = np.real(Psi * np.conj(Psi)) * 1.5  # psi squared, probability(x)
    wpacket.points(np.c_[x, amp, zeros])     # update points
    plt.render()

plt.interactive().close()



"""Using 1D Moving Least Squares to skeletonize a surface"""
from vedo import dataurl, Points, Plotter

N = 9    # nr of iterations
f = 0.2  # fraction of neighbours

pcl = Points(dataurl+"man.vtk").subsample(0.02)

plt = Plotter(N=N, axes=1)
for i in range(N):
    pcl = pcl.clone().smooth_mls_1d(f=f).color(i)
    plt.at(i).show(f"iteration {i}", pcl, elevation=-5)

plt.interactive().close()



"""Compute the (signed) distance of one mesh to another"""
from vedo import Sphere, Cube, show

# Create a sphere object and position it at (10,20,30)
s1 = Sphere().pos(10,20,30)

# Create a cube object with color grey and scaled 
# along the x-axis by 2, and positioned at (14,20,30)
s2 = Cube(c='grey4').scale([2,1,1]).pos(14,20,30)

# Compute the Euclidean distance between the 2 surfaces
# and set the color of the sphere based on the distance
s1.distance_to(s2, signed=False)
s1.cmap('hot').add_scalarbar('Signed\nDistance')

# Show the sphere, the cube, the script docstring, axes,
# then close the window
show(s1, s2, __doc__ , axes=1, size=(1000,500), zoom=1.5).close()



"""Extract all image voxels as points"""
from vedo import *

v = Volume(dataurl+'vase.vti')

pts = v.topoints().print() # returns Points

scalars = pts.pointdata[0]
pts.cmap('afmhot_r', scalars).point_size(1)

show([(v,__doc__), pts], N=2, viewup='z', bg2='lightblue', axes=1).close()



"""Warp scalars inside of a Volumetric dataset"""
from vedo import Volume, Cube, Arrows, show, dataurl

vol = Volume(dataurl+"embryo.tif")

source = Cube().scale(3000)
target = Cube().scale([4000,5000,6000]).rotate_x(20).wireframe()

arrs = Arrows(source, target, c='k')

vol.warp(source, target, fit=True)

show(vol, arrs, source, target, __doc__, axes=1, viewup='z')



"""Visualization of a discretized Earth model"""
import vedo

vedo.settings.default_font = 'Kanopus'

tet = vedo.TetMesh(vedo.dataurl+'earth_model.vtu')
conductor = tet.clone().threshold('cell_scalars', above=0, below=4)

# Crop the initial mesh
box = vedo.Box(size=[503500, 505000, 6414000, 6417000, -1830, 600])
tet.cut_with_mesh(box, whole_cells=True)

# We need to build a look up table for our color bar
lut_table = [
    #value, color,   alpha, category_label
    ( 0.0, 'black',      1, "Cond_0"),
    ( 1.0, 'cyan',       1, "Cond_1"),
    ( 2.0, 'skyblue',    1, "Cond_2"),
    ( 3.0, 'dodgerblue', 1, "Cond_3"),
    ( 4.0, 'blue',       1, "Cond_4"),
    ( 5.0, 'gray',       1, "Overburden"),
    ( 6.0, 'yellow',     1, "Layer^A"),
    ( 7.0, 'gold',       1, "Layer^B"),
    ( 9.0, 'red',        1, "Layer^C"),
    (11.0, 'powderblue', 1, "Layer^D"),
    (13.0, 'lime',       1, "Layer^E"),
    (15.0, 'seagreen',   1, "Layer^V"),
]
lut = vedo.build_lut(lut_table)

msh = tet.tomesh(shrink=0.95).cmap(lut, 'cell_scalars', on='cells')
msh.add_scalarbar3d(
    categories=lut_table,
    pos=(505500, 6416900, -630),
    title='Units',
    title_size=1.25,
    label_size=1.5,
    size=[100, 2200],
)
# put scalarbar vertical, tell camera to keep bounds into account
msh.scalarbar.rotate_x(90, around='itself').rotate_z(60, around='itself')
msh.scalarbar.use_bounds()

# Create cmap for conductor
cond = conductor.tomesh().cmap(lut, 'cell_scalars', on='cells')

axes = vedo.Axes(
    msh + cond,
    xtitle='Easting (m)',
    ytitle='Northing (m)',
    ztitle='Elevation (m)',
    xtitle_position=0.65,
    ytitle_position=0.65,
    ztitle_position=0.65,
    ytitle_offset=-0.22,
    ztitle_offset= 0.06,
    ylabel_rotation=90,
    ylabel_offset=-1.5,
    zaxis_rotation=15,
    axes_linewidth=3,
    grid_linewidth=2,
    yshift_along_x=1,
    tip_size=0,
    yzgrid=True,
    xyframe_line=True,
)

vedo.show(msh, cond, axes, __doc__, size=(1305, 1020),
          roll=-80, azimuth=50, elevation=-10, zoom=1.25).close()



"""A simple quiver plot"""
from vedo import Grid, Arrows2D, show


# Create displacements
pts1 = Grid(s=[1.0,1.0]).points()
pts2 = Grid(s=[1.2,1.2]).rotate_z(4).points()

quiv = Arrows2D(pts1, pts2, c="red5")

show(quiv, __doc__, axes=1, zoom=1.2).close()



"""Probe a voxel dataset at specified points
and plot a histogram of the values"""
from vedo import np, dataurl, Volume, Axes, probe_points, show
from vedo.pyplot import histogram

vol = Volume(dataurl+'embryo.slc').print()

pts = np.random.rand(5000, 3)*256
mpts = probe_points(vol, pts).point_size(3)

mpts.print()
# valid = mpts.pointdata['vtkValidPointMask']
scals = mpts.pointdata['SLCImage']

his = histogram(scals, xtitle='probed voxel value', xlim=(5,100))

show([(vol, Axes(vol), mpts, __doc__), his], N=2, sharecam=False).close()



"""Interpolate a vectorial field using
Thin Plate Spline or Radial Basis Function"""
from scipy.interpolate import Rbf
from vedo import Plotter, Points, Arrows, show
import numpy as np


ls = np.linspace(0, 10, 8)
X, Y, Z = np.meshgrid(ls, ls, ls)
xr, yr, zr = X.ravel(), Y.ravel(), Z.ravel()
positions = np.vstack([xr, yr, zr])

sources = [(5, 8, 5), (8, 5, 5), (5, 2, 5)]
deltas = [(1, 1, 0.2), (1, 0, -0.8), (1, -1, 0.2)]

apos = Points(positions, r=2)

# for p in apos.points(): ####### Uncomment to fix some points.
#    if abs(p[2]-5) > 4.999:  # differences btw RBF and thinplate
#        sources.append(p)    # will become much smaller.
#        deltas.append(np.zeros(3))
sources = np.array(sources)
deltas = np.array(deltas)

src = Points(sources, c="r", r=12)
trs = Points(sources + deltas, c="v", r=12)
arr = Arrows(sources, sources + deltas).color("k8")

################################################# warp using Thin Plate Splines
warped = apos.clone().warp(sources, sources+deltas)
warped.alpha(0.4).color("lg").point_size(10)
allarr = Arrows(apos.points(), warped.points()).color("k8")

set1 = [apos, warped, src, trs, arr, __doc__]
plt1 = show([set1, allarr], N=2, bg='bb', interactive=0)  # returns the Plotter class


################################################# RBF
x, y, z = sources[:, 0], sources[:, 1], sources[:, 2]
dx, dy, dz = deltas[:, 0], deltas[:, 1], deltas[:, 2]

itrx = Rbf(x, y, z, dx)  # Radial Basis Function interpolator:
itry = Rbf(x, y, z, dy)  #  interoplate the deltas in each separate
itrz = Rbf(x, y, z, dz)  #  cartesian dimension

positions_x = itrx(xr, yr, zr) + xr
positions_y = itry(xr, yr, zr) + yr
positions_z = itrz(xr, yr, zr) + zr
positions_rbf = np.vstack([positions_x, positions_y, positions_z])

warped_rbf = Points(positions_rbf, r=2).alpha(0.4).color("lg").point_size(10)
allarr_rbf = Arrows(apos.points(), warped_rbf.points()).color("k8")

arr = Arrows(sources, sources + deltas).color("k8")

plt2 = Plotter(N=2, pos=(200, 300), bg='bb')
plt2.at(0).show("Radial Basis Function", apos, warped_rbf, src, trs, arr)
plt2.at(1).show(allarr_rbf)
plt2.interactive()

plt2.close()
plt1.close()

